
///////////////////////////////////////////////////////////////////////////
// This program is free software: you can redistribute it and/or modify  //
// it under the terms of the version 3 of the GNU General Public License //
// as published by the Free Software Foundation.                         //
//                                                                       //
// This program is distributed in the hope that it will be useful, but   //
// WITHOUT ANY WARRANTY; without even the implied warranty of            //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      //
// General Public License for more details.                              //
//                                                                       //
// You should have received a copy of the GNU General Public License     //
// along with this program. If not, see <http://www.gnu.org/licenses/>.  //
//                                                                       //
// Written by Francois Fleuret                                           //
// (C) Idiap Research Institute                                          //
//                                                                       //
// Contact <francois.fleuret@idiap.ch> for comments & bug reports        //
///////////////////////////////////////////////////////////////////////////

/*

  Same as RGBImage, but with sub-pixel resolution to produce nifty
  smoothing in the drawings.

 */

#ifndef RGB_IMAGE_SUBPIXEL_H
#define RGB_IMAGE_SUBPIXEL_H

#include "misc.h"
#include "rgb_image.h"

class RGBImageSubpixel : public RGBImage {
  static const int _scale = 8;
public:

  RGBImageSubpixel(int width, int height);
  RGBImageSubpixel(RGBImage *image);
  virtual ~RGBImageSubpixel();

  inline int width() const { return _width / _scale; }
  inline int height() const { return _height / _scale; }

  virtual void read_png(const char *filename);
  virtual void write_png(const char *filename);

  virtual void read_jpg(const char *filename);
  virtual void write_jpg(const char *filename, int quality);

  virtual void draw_line(int thickness,
                         unsigned char r, unsigned char g, unsigned char b,
                         scalar_t x0, scalar_t y0, scalar_t x1, scalar_t y1);
};

#endif
